/* *************************************************************************
 
   								Millstone(TM) 
   				   Open Sourced User Interface Library for
   		 		       Internet Development with Java

             Millstone is a registered trademark of IT Mill Ltd
                  Copyright (C) 2000-2005 IT Mill Ltd
                     
   *************************************************************************

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   license version 2.1 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   *************************************************************************
   
   For more information, contact:
   
   IT Mill Ltd                           phone: +358 2 4802 7180
   Ruukinkatu 2-4                        fax:  +358 2 4802 7181
   20540, Turku                          email: info@itmill.com
   Finland                               company www: www.itmill.com
   
   Primary source for MillStone information and releases: www.millstone.org

   ********************************************************************** */

package com.itmill.tk.ui;

import com.itmill.tk.data.BufferedValidatable;
import com.itmill.tk.data.Property;
import com.itmill.tk.ui.Component.Focusable;

/**
 * @author Sami Ekblad
 *
 */
public interface Field
	extends
		Component,
		BufferedValidatable,
		Property,
		Property.ValueChangeNotifier,
		Property.ValueChangeListener,
		Property.Editor,
		Focusable {

	void setCaption(String caption);

	String getDescription();
	
	void setDescription(String caption);

	/** Is this field required.
	 * 
	 * Required fields must filled by the user.
	 * 
	 * @return true if the
	 * @since 3.1 
	 */
	public boolean isRequired();

	/** Set the field required.
	 * Required fields must filled by the user.
	 * 
	 * @param required Is the field required
	 * @since 3.1 
	 */
	public void setRequired(boolean required);

	/** An <code>Event</code> object specifying the Field whose value
	 * has been changed.
	 * @author IT Mill Ltd.
		 * @version @VERSION@
		 * @since 3.0
	 */
	public class ValueChangeEvent
		extends Component.Event
		implements Property.ValueChangeEvent {

		/**
         * Serial generated by eclipse.
         */
        private static final long serialVersionUID = 3545803169444672816L;

        /** Constructs a new event object with the specified source
		 * field object.
		 *
		 * @param source the field that caused the event
		 */
		public ValueChangeEvent(Field source) {
			super(source);
		}

		/** Gets the Property which triggered the event.
		 *
		 * @return Source Property of the event.
		 */
		public Property getProperty() {
			return (Property) getSource();
		}
	}
}
